'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var tslib_1 = require('tslib');
var React = require('react');
var reactSpring = require('react-spring');
var reactGestureResponder = require('react-gesture-responder');
var useScrollLock = _interopDefault(require('use-scroll-lock'));

function usePrevious(value) {
    var ref = React.useRef(null);
    React.useEffect(function () {
        ref.current = value;
    }, [value]);
    return ref.current;
}

function Pane(_a) {
    var children = _a.children, active = _a.active, width = _a.width, rotate = _a.rotate, height = _a.height, style = _a.style, other = tslib_1.__rest(_a, ["children", "active", "width", "rotate", "height", "style"]);
    var ref = React.useRef(null);
    React.useEffect(function () {
        if (active && ref.current) {
            ref.current.focus();
        }
    }, [active]);
    return (React.createElement("div", tslib_1.__assign({ className: "Cube__Pane", "aria-hidden": !active, tabIndex: active ? 0 : -1, ref: ref, style: tslib_1.__assign({ display: "flex", flexDirection: "column", backfaceVisibility: "visible", outline: "none", padding: "3px", position: "absolute", width: width + "px", height: height + "px", transform: "rotateY(" + rotate + "deg) translateZ(" + width / 2 + "px)" }, style) }, other), children));
}

/**
 * React-Cube
 *
 * A 3d rotated cube which supports gestures
 * and an infinite number of views (i.e., it supports
 * windowing). It's inspired by the instagram story
 * inteface.
 */
function Cube(_a) {
    var _b = _a.hasNext, hasNext = _b === void 0 ? function () { return true; } : _b, onChange = _a.onChange, providedIndex = _a.index, renderItem = _a.renderItem, _c = _a.width, width = _c === void 0 ? 200 : _c, _d = _a.height, height = _d === void 0 ? 600 : _d, _e = _a.perspective, perspective = _e === void 0 ? 800 : _e, paneStyle = _a.paneStyle, _f = _a.scaleRange, scaleRange = _f === void 0 ? [1, 0.95] : _f, _g = _a.lockScrolling, lockScrolling = _g === void 0 ? false : _g, _h = _a.enableGestures, enableGestures = _h === void 0 ? true : _h;
    // optionally allow the user to pass in an object
    // with an index and animated property. This allows
    // the user to skip to an index without animating.
    var _j = typeof providedIndex === "number"
        ? { immediate: false, index: providedIndex }
        : providedIndex, immediate = _j.immediate, index = _j.index;
    var _k = reactSpring.useSpring(function () { return ({
        rotateY: index * -90,
        immediate: true
    }); }), props = _k[0], set = _k[1];
    var prevIndex = usePrevious(index);
    var currentActivePane = index % 4;
    // this is a mess... gotta refactor. basically it determines
    // what the initial render indexes should be given the
    // initial index to show
    var _l = React.useState(function () {
        return resetIndexes(index, true);
    }), indexesToRender = _l[0], setIndexesToRender = _l[1];
    var _m = React.useState(false), animating = _m[0], setAnimating = _m[1];
    // lock body scrolling when gesturing or animating
    useScrollLock(animating || lockScrolling);
    /**
     * Animate our cube into position when our active
     * index changes
     */
    function onRest() {
        setAnimating(false);
    }
    React.useEffect(function () {
        setAnimating(true);
        set({ rotateY: index * 90 * -1, immediate: immediate, onRest: onRest });
    }, [onRest, index, set, immediate]);
    /**
     * On drag end, determine the index to show
     */
    var onEnd = React.useCallback(function (_a) {
        var delta = _a.delta, velocity = _a.velocity;
        if (velocity > 0.12) {
            onChange(delta[0] < 0 ? index + 1 : index - 1);
            return;
        }
        // next
        if (delta[0] < -(width / 2)) {
            onChange(index + 1);
            // prev
        }
        else if (delta[0] > width / 2) {
            onChange(index - 1);
            // current
        }
        else {
            set({ rotateY: index * 90 * -1, onRest: function () { }, immediate: false });
        }
    }, [index, width]);
    /**
     * Support basic drag gestures
     */
    var bind = reactGestureResponder.useGestureResponder({
        onStartShouldSet: function () { return false; },
        onMoveShouldSet: function (_a) {
            var direction = _a.direction;
            if (!enableGestures) {
                return false;
            }
            // beginning of stack and swiping left
            if (index === 0 && direction[0] > 0) {
                return false;
            }
            // end of stack and swiping right
            if (direction[0] < 0 && !hasNext(index)) {
                return false;
            }
            // swiping horizontally
            return Math.abs(direction[0]) > Math.abs(direction[1]);
        },
        onRelease: onEnd,
        onTerminate: onEnd,
        onMove: function (_a) {
            var delta = _a.delta;
            if (!animating) {
                setAnimating(true);
            }
            var currentRotate = index * 90 * -1;
            var convert = linearConversion([0, width], [currentRotate, currentRotate + 90]);
            var v = convert(delta[0]);
            if (v > currentRotate + 90) {
                v = currentRotate + 90;
            }
            else if (v < currentRotate - 90) {
                v = currentRotate - 90;
            }
            set({
                rotateY: v,
                immediate: true,
                onRest: function () { }
            });
        }
    }).bind;
    /**
     * Handle rendering of individual panes.
     */
    React.useEffect(function () {
        // todo: usecallback style updates
        if (prevIndex && prevIndex !== index) {
            setIndexesToRender(resetIndexes(index, index > prevIndex, indexesToRender.slice()));
        }
    }, [prevIndex, indexesToRender, index, immediate]);
    return (React.createElement(reactSpring.animated.div, tslib_1.__assign({ className: "Cube", style: {
            width: width + "px",
            height: height + "px",
            transform: props.rotateY.to(function (x) { return "scale(" + getScale(x, scaleRange); }),
            perspective: perspective + "px"
        } }, bind),
        React.createElement(reactSpring.animated.div, { className: "Cube__animated-container", style: {
                width: "100%",
                height: "100%",
                position: "relative",
                backfaceVisibility: "visible",
                transformStyle: "preserve-3d",
                transform: props.rotateY.to(function (x) { return "translateZ(-" + width / 2 + "px) rotateY(" + x + "deg)"; })
            } },
            React.createElement(Pane, { width: width, height: height, rotate: -90, active: currentActivePane === 3, style: paneStyle }, renderItem(indexesToRender[3], currentActivePane === 3 && !animating, props.rotateY)),
            React.createElement(Pane, { width: width, height: height, rotate: 0, active: currentActivePane === 0, style: paneStyle }, renderItem(indexesToRender[0], currentActivePane === 0 && !animating, props.rotateY)),
            React.createElement(Pane, { width: width, height: height, rotate: 90, active: currentActivePane === 1, style: paneStyle }, renderItem(indexesToRender[1], currentActivePane === 1 && !animating, props.rotateY)),
            React.createElement(Pane, { width: width, height: height, rotate: -180, active: currentActivePane === 2, style: paneStyle }, renderItem(indexesToRender[2], currentActivePane === 2 && !animating, props.rotateY)))));
}
function getScale(x, scaleRange) {
    var diff = Math.abs(x) % 90;
    if (!diff) {
        return 1;
    }
    var convert = diff > 45
        ? linearConversion([90, 45], [1, 0.5])
        : linearConversion([45, 0], [0.5, 1]);
    var eased = circ(convert(diff));
    var scale = linearConversion([1, 0.5], scaleRange);
    return scale(eased);
}
function linearConversion(a, b) {
    var o = a[1] - a[0], n = b[1] - b[0];
    return function (x) {
        return ((x - a[0]) * n) / o + b[0];
    };
}
function circ(timeFraction) {
    return 1 - Math.sin(Math.acos(timeFraction));
}
function resetIndexes(index, forward, indexes) {
    if (indexes === void 0) { indexes = [-1, -1, -1, -1]; }
    var currentActivePane = index % 4;
    indexes[currentActivePane] = index;
    var prevIndex = getPreviousIndex(currentActivePane - 1);
    indexes[prevIndex] = index - 1;
    var nextIndex = getNextIndex(currentActivePane + 1);
    indexes[nextIndex] = index + 1;
    if (forward) {
        indexes[getNextIndex(nextIndex + 1)] = index + 2;
    }
    else {
        indexes[getPreviousIndex(index - 1)] = index - 2 < -1 ? -1 : index - 2;
    }
    return indexes;
}
function getPreviousIndex(i) {
    if (i > -1)
        return i;
    return 3;
}
function getNextIndex(i) {
    if (i < 4)
        return i;
    return 0;
}

exports.Cube = Cube;
exports.Pane = Pane;
exports.default = Cube;
//# sourceMappingURL=index.js.map
